'' =================================================================================================
''
''   File....... jm_countdown_timer.spin
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2015 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 05 DEC 2015
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                           ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                          ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                    ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                                ' ticks in 1us


con { io pins }

  RX1 = 31                                                       ' programming / terminal
  TX1 = 30

  SDA = 29                                                       ' i2c / eeprom
  SCL = 28


con

  TMR_SECS = 90                                                  ' 90-second countdown
  

obj

' main                                                           ' * master Spin cog
  time    : "jm_time"                                            '   timing and delays
  io      : "jm_io_basic"                                        '   essential io control
  term    : "jm_fullduplexserial"                                ' * serial io for terminal

' * Uses cog when loaded


var

  long  last                                                     ' last displayed time (change flag)
  long  now                                                      ' current time


pub main 

  setup

  term.rxflush                                                   ' wait open terminal & keypress
  term.rx
  term.tx(term#CLS)

  time.set_secs(-TMR_SECS)                                       ' start timing

  repeat
    check_time                                                   ' check for timer change

    ' other loop code here


pub check_time | nowms

'' Checks timer for 1s change
'' -- if change, time display is updated
'' -- if time expired, stops the timer

  nowms := time.millis                                           ' capture time
  now := -nowms / 1000                                           ' extract millis (remove sign)
  if (nowms // 1000)                                             ' if fractional seconds  
    ++now                                                        '  round up for display  

  if (now <> last)                                               ' changed?
    format_time 
    last := now                                                  '  mark change
    
  if (now == 0)                                                  ' done?
    time.hold
    

pub format_time | m, s

'' Display formatted time as mm:ss

  term.tx(term#HOME)

  m := now  / 60                                                 ' extract minutes
  s := now // 60                                                 ' extract seconds

  term.tx("0" + m  / 10)
  term.tx("0" + m // 10)
  term.tx(":")
  term.tx("0" + s  / 10)
  term.tx("0" + s // 10)

  
pub setup

'' Setup IO and objects for application

  time.start                                                     ' setup timing/delays 

  io.start(0, 0)                                                 ' clear all pins

  term.start(RX1, TX1, %0000, 115_200)                           ' start serial for terminal


dat { license }

{{

  Copyright (C) 2015 Jon McPhalen

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}